// (C) Copyright 2013-2019 NuoDB, Inc.  All Rights Reserved.

#pragma once

// Modifications to this list must also update Java's SQLState.java &
// CDriver/nuodb/error.h, and it would sure be handy if they also
// got made in protocol.py in nuodb-python

/**
 * @namespace NuoDB::NuoDBSqlConstants
 * @brief Sub-namespace for the SQL exception codes used by SQLException.
 */

#define SQL_CODES \
    FORMAT(-1,  SYNTAX_ERROR) \
            /**< SQL syntax error, Example: "wrong number of arguments for
            function" */ \
    FORMAT(-2,  FEATURE_NOT_YET_IMPLEMENTED) \
            /**< Feature is not yet implemented */ \
    FORMAT(-3,  BUG_CHECK) \
            /**< An internal database error has occurred */ \
    FORMAT(-4,  COMPILE_ERROR) \
            /**< An error has occurred during compilation of the SQL statement,
            Example: "inconsistent select list in union branches" */ \
    FORMAT(-5,  RUNTIME_ERROR) \
            /**< An error has occurred during excecution of an SQL statement,
            Example: "ResultSet has been closed" */ \
    FORMAT(-6,  OCS_ERROR) \
            /**< Unused */ \
    FORMAT(-7,  NETWORK_ERROR) \
            /**< A network communication has occurred, either with the Broker
            or TE, Example: "no nodes are available for database" */ \
    FORMAT(-8,  CONVERSION_ERROR) \
            /**< An conversion error has occurred during execution of an SQL
            statement, Example: "Numeric overflow converting..." */ \
    FORMAT(-9,  TRUNCATION_ERROR) \
            /**< A truncation error has occurred during execution of an SQL
            statement, Example: "Assignment of n bytes into type of maximum
            length m" */ \
    FORMAT(-10, CONNECTION_ERROR) \
            /**< Error related to making an SQL connection or to an already
             existing SQL connection, Examples: "database already open",
             "bad statement handle" */ \
    FORMAT(-11, DDL_ERROR) \
            /**< A runtime error has occurred during execution of an SQL DDL
            statement, Example: "table already defined" */ \
    FORMAT(-12, APPLICATION_ERROR) \
            /**< An application specific error has occurred, Example: "Cannot
            execute PreparedStatement with SQL parameter"*/ \
    FORMAT(-13, SECURITY_ERROR) \
            /**< A database security error has occurred, such as bad credentials,
            Examples: "no username specified", "not a known user", "requested
            access to foo is denied", "Unsupported cipher requested"*/ \
    FORMAT(-14, DATABASE_CORRUPTION) \
            /**< Unused */ \
    FORMAT(-15, VERSION_ERROR) \
            /**< Unused */ \
    FORMAT(-16, LICENSE_ERROR) \
            /**< Unused */ \
    FORMAT(-17, INTERNAL_ERROR) \
            /**< An internal error has occurred. This should never happen.
            Examples: "function hasn't been compiled", "Unexpected NULL lower
            bound node" */ \
    FORMAT(-18, DEBUG_ERROR) \
            /**< Unused */ \
    FORMAT(-19, LOST_BLOB) \
            /**< Unused */ \
    FORMAT(-20, INCONSISTENT_BLOB) \
            /**< Unused */ \
    FORMAT(-21, DELETED_BLOB) \
            /**< Unused */ \
    FORMAT(-22, LOG_ERROR) \
            /**< Unused */ \
    FORMAT(-23, DATABASE_DAMAGED) \
            /**< Unused */ \
    FORMAT(-24, UPDATE_CONFLICT) \
            /**< Server error has occurred, Example: "pending update
            rejected, transaction nnn" */ \
    FORMAT(-25, NO_SUCH_TABLE) \
            /**< An invalid table reference has occurred, Examples: "Table has
            been dropped", "Can't find table" */ \
    FORMAT(-26, INDEX_OVERFLOW) \
            /**< Unused */ \
    FORMAT(-27, UNIQUE_DUPLICATE) \
            /**< Server error regarding duplicate values in a unique index */ \
    FORMAT(-28, UNCOMMITTED_UPDATES) \
            /**< ??? */ \
    FORMAT(-29, DEADLOCK) \
            /**< Server deadlock error */ \
    FORMAT(-30, OUT_OF_MEMORY_ERROR) \
            /**< Unused */ \
    FORMAT(-31, OUT_OF_RECORD_MEMORY_ERROR) \
            /**< Unused */ \
    FORMAT(-32, LOCK_TIMEOUT) \
            /**< Statement timed out */ \
    FORMAT(-36, PLATFORM_ERROR) \
            /**< Generic server error */ \
    FORMAT(-37, NO_SCHEMA) \
            /**< An error occurred because no schema specified where schema is
            required. */ \
    FORMAT(-38, CONFIGURATION_ERROR) \
            /**< Server configuration error, Example: "Could not find a
            valid archive at the given location" */ \
    FORMAT(-39, READ_ONLY_ERROR) \
            /**< An error occurred where an attempt was made to write to a read
            only connection. */ \
    FORMAT(-40, NO_GENERATED_KEYS) \
            /**< Unused */ \
    FORMAT(-41, THROWN_EXCEPTION) \
            /**< This is used by the SQL THROW control statement */ \
    FORMAT(-42, INVALID_TRANSACTION_ISOLATION) \
            /**< Unused */ \
    FORMAT(-43, UNSUPPORTED_TRANSACTION_ISOLATION) \
            /**< An error occurred where the transaction level specified for
            connection is unsupported */ \
    FORMAT(-44, INVALID_UTF8) \
            /**< An error occurred where a string that is expected to be UTF-8
            is not */ \
    FORMAT(-45, CONSTRAINT_ERROR) \
            /**< A SQL constraint violation has occurred */ \
    FORMAT(-46, UPDATE_ERROR) \
            /**< An attempt was made to update a database object that cannot be
            updated, Example: "Table returned by a stored procedure cannot be
            manipulated" */ \
    FORMAT(-47, I18N_ERROR) \
            /**< An error having to do with Internationalization has occurred,
            Example: "Error getting default converter" */ \
    FORMAT(-48, OPERATION_KILLED) \
            /**< A SQL operation was killed, perhaps by the KILL STATEMENT
            command */ \
    FORMAT(-49, INVALID_STATEMENT) \
            /**< An invalid reference to a SQL statement occurred, Example: "On
            connection n, Unable to kill statement m. No such statement id" */ \
    FORMAT(-50, IS_SHUTDOWN) \
            /**< An error occurred because the database is shutting down. */ \
    FORMAT(-51, IN_QUOTED_STRING) \
            /**< An internal error generated by parser when the statement string
            ends within a quoted string (used by nuosql) */ \
    FORMAT(-52, BATCH_UPDATE_ERROR) \
            /**< Catchall for Batch update exceptions */ \
    FORMAT(-53, JAVA_ERROR) \
            /**< An error has occurred, related to Java stored procedures */ \
    FORMAT(-54, INVALID_FIELD) \
            /**< A server error setting a value to an invalid field */ \
    FORMAT(-55, INVALID_INDEX_NULL) \
            /**< An invalid null was inserted into index */ \
    FORMAT(-56, INVALID_OPERATION) \
            /**< A generic error related to an invalid operation, Examples:
            "Maximum number of open result sets exceeded", "Maximum number of
            open statements exceeded" */ \
    FORMAT(-57, INVALID_STATISTICS) \
            /**< Index statistics for a specified query have become invalid */ \
    FORMAT(-58, INVALID_GENERATOR) \
            /**< Badly defined generator defined in older version of nuodb. Need to
             ALTER TABLE to change the columns type to either INT, BIGINT, NUMERIC or
             STRING */ \
    FORMAT(-59, OPERATION_TIMEOUT) \
            /**< Query timeout period expired before a statement object executed.
             * Examples: Timeout of n micro second(s) exceeded", "Timeout of n second(s) exceeded"
             */ \
    FORMAT(-60, NO_SUCH_INDEX) \
            /**< An invalid index reference has occurred, Examples: "Index has
            been dropped", "Can't index table" */ \
    FORMAT(-61, NO_SUCH_SEQUENCE) \
            /**< An invalid sequence reference has occurred, Examples: "Sequence has
            been dropped", "Sequence doesn't exist" */ \
    FORMAT(-62, XAER_PROTO) \
            /**< An error with the XA protocol has occurred, Examples:
             * "TransactionId already prepared" */ \
    FORMAT(-63, UNKNOWN_ERROR) \
            /**< An unknown error code
             * An unknown error code
             */ \
    FORMAT(-64, TRANSACTIONAL_LOCK_ERROR) \
            /**< An error occurred acquiring a transactional lock
             */ \
    FORMAT(-65, TRANSACTION_UNKNOWN_STATE) \
            /** TE shutdown was requested before the transaction successfully committed
             */

// Modifications to this mapping must also update Java's SQLState.java
#define NUODB_CODES_TO_SQLSTATES  \
    FORMAT( -1 , 42000 ) \
    FORMAT( -2 , 0A000 ) \
    FORMAT( -3 , 58000 ) \
    FORMAT( -4 , 42000 ) \
    FORMAT( -5 , 58000 ) \
    FORMAT( -6 , 08000 ) \
    FORMAT( -7 , 08000 ) \
    FORMAT( -8 , 22000 ) \
    FORMAT( -9 , 22000 ) \
    FORMAT( -10, 08000 ) \
    FORMAT( -11, 42000 ) \
    FORMAT( -12, 58000 ) \
    FORMAT( -13, 58000 ) \
    FORMAT( -14, 58000 ) \
    FORMAT( -15, 58000 ) \
    FORMAT( -16, 58000 ) \
    FORMAT( -17, 58000 ) \
    FORMAT( -18, 58000 ) \
    FORMAT( -19, 22000 ) \
    FORMAT( -20, 22000 ) \
    FORMAT( -21, 22000 ) \
    FORMAT( -22, 58000 ) \
    FORMAT( -23, 58000 ) \
    FORMAT( -24, 40002 ) \
    FORMAT( -25, 42000 ) \
    FORMAT( -26, 58000 ) \
    FORMAT( -27, 23000 ) \
    FORMAT( -28, 58000 ) \
    FORMAT( -29, 40001 ) \
    FORMAT( -30, 58000 ) \
    FORMAT( -31, 58000 ) \
    FORMAT( -32, 58000 ) \
    FORMAT( -36, 58000 ) \
    FORMAT( -37, 58000 ) \
    FORMAT( -38, 58000 ) \
    FORMAT( -39, 58000 ) \
    FORMAT( -40, 58000 ) \
    FORMAT( -41, 58000 ) \
    FORMAT( -42, 01000 ) \
    FORMAT( -43, 0A000 ) \
    FORMAT( -44, 58000 ) \
    FORMAT( -45, 23001 ) \
    FORMAT( -46, 58000 ) \
    FORMAT( -47, 58000 ) \
    FORMAT( -48, HY008 ) \
    FORMAT( -49, 58000 ) \
    FORMAT( -50, 58000 ) \
    FORMAT( -51, 58000 ) \
    FORMAT( -52, 58000 ) \
    FORMAT( -53, 58000 ) \
    FORMAT( -54, 58000 ) \
    FORMAT( -55, 58000 ) \
    FORMAT( -56, 58000 ) \
    FORMAT( -57, 58000 ) \
    FORMAT( -58, 58000 ) \
    FORMAT( -59, HYT00 ) \
    FORMAT( -60, 42S12 ) \
    FORMAT( -61, HY101 ) \
    FORMAT( -62, HY102 ) \
    FORMAT( -63, HYZZZ ) \
    FORMAT( -64, 40003 ) \
    FORMAT( -65, 08007 ) \

// modifications to this list must also update Java's SQLState.java and to
// NUODB_CODES_TO_SQLSTATE mapping defined below and, it would sure be handy
// if they also got made in protocol.py in nuodb-python

namespace NuoDB
{

namespace NuoDBSqlConstants
{

/*
 * @brief   Enum for codes set in an SQLException object to tell what type of
 *          exception has been thrown.
 */
enum SqlCode{
#define FORMAT(ERR_NUM, ERR_STRING) ERR_STRING = ERR_NUM,
    SQL_CODES
#undef FORMAT
};

/**
 * Function to retrieve string reperesenting error code.
 */
inline const char* codeToErrorString(int code) {
    switch(code) {
#define FORMAT(ERR_NUM, ERR_STRING) case ERR_NUM: return #ERR_STRING;
        SQL_CODES
#undef FORMAT
        default: return "NOT_A_CODE";
    }
}

/**
 * Function to retrieve string reperesenting SQLSTATE.
 */
inline const char* nuoDBCodeToSQLSTATE(int code) {
    switch (code) {
#define FORMAT(NUODB_CODE, SQLSTATE) case NUODB_CODE: return #SQLSTATE;
        NUODB_CODES_TO_SQLSTATES
#undef FORMAT
        default: return "HY000";
    }
}

}

using namespace NuoDBSqlConstants;
}
