/* Copyright 2015 NuoDB, Inc. All Rights Reserved. */
#ifndef _NUODB_DATABASEMETADATA_H_
#define _NUODB_DATABASEMETADATA_H_ 1

/* c-driver */
#ifdef __cplusplus
extern "C" {
#endif

/**
 * @file databaseMetaData.h
 * @brief Metadata about a NuoDB database.
 */

/*! Metadata about a %NuoDB database.
 *
 * NuoDB_DatabaseMetaData is used to obtain information about a %NuoDB
 * database.
 */
struct NuoDB_DatabaseMetaData
{
    _NuoDB_DatabaseMetaData_state_t* _state;

    /**
     * @brief   Returns the connection URL string
     *
     * @param[in]   _this   a pointer to the current (this)
     * NuoDB_DatabaseMetaData data structure.
     *
     * @return  a pointer to the connection string.
     */
    const char* (* getConnectionURL)(NuoDB_DatabaseMetaData* _this);

    /**
     * @brief   Returns the Database Product Name string
     *
     * @param[in]   _this   a pointer to the current (this)
     * NuoDB_DatabaseMetaData data structure.
     *
     * @return  a pointer to the Database Product Name or NULL if an error.
     */
    const char* (* getDatabaseProductName)(NuoDB_DatabaseMetaData* _this);

    /**
     * @brief   Returns Database Product Version string
     *
     * @param[in]   _this   a pointer to the current (this)
     * NuoDB_DatabaseMetaData data structure.
     *
     * @return  a pointer to the database product version string or
     * NULL if an error.
     */
    const char* (* getDatabaseProductVersion)(NuoDB_DatabaseMetaData* _this);

    /**
     * @brief   Returns driver major version number
     *
     * @param[in]   _this   a pointer to the current (this)
     * NuoDB_DatabaseMetaData data structure.
     *
     * @return  a integer for the driver major version number.
     */
    int (* getDriverMajorVersion)(NuoDB_DatabaseMetaData* _this);

    /**
     * @brief   Returns driver minor version number
     *
     * @param[in]   _this   a pointer to the current (this)
     * NuoDB_DatabaseMetaData data structure.
     *
     * @return  a integer for the driver minor version number.
     */
    int (* getDriverMinorVersion)(NuoDB_DatabaseMetaData* _this);
};

#ifdef __cplusplus
} /* extern "C" { */
#endif

#endif /* end of: #ifndef _NUODB_DATABASEMETADATA_H_ */
